; example for the controller-board (www.frydaysoft.de)
;flash one Leds and put pressed KEY to LED
;poll the HCT245 and put this data to HCT574
;put text to LCD-display

$INCLUDE (C:\ide51\inc\mod51)

;**** Bit-RAM 20..2F ****
 BSEG AT 02Fh			; use only last byte of BSEG-block     
  blink_cursor: DBIT 1	; param for init_lcd
  step_walk:    DBIT 1	; param for writeLCD_Array

;**** Byte-RAM from 8h upto 2Dh ****
 LCD_BUFMAX		EQU 10h
 LCD_BUFFER_END EQU LCD_BUFMAX+1
 DSEG AT 8h					; bank1..3 8..1Fh
  lcd_buffer:  DS LCD_BUFMAX	; string-buffer 16 byte for LCD-Output
  unused_buf:  DS  8h		; 8 byte unused and BSEG not complete used
 DSEG AT 30h					; 30..7Fh
  STACK_RAM:   DS 50h		; **** reserved STACK-area  ****


;**** Reg-zuweisung ****
Key4	SET	P1.0
Key3	SET	P1.1
Key2	SET	P1.2
Key1	SET	P1.3
Led4	SET	P1.4
Led3	SET	P1.5
Led2	SET	P1.6
Led1	SET	P1.7

tmppointer	EQU R0
;R1..R7 not used

waitconst 	EQU   0h		;const for wait-looping
Stack_First EQU  30h		;set stackspointer to reserved RAM

;LCD-DDRAM-Adress
;Type			1. Line		2. Line
;1x8			0..7h
;1x16 (8+8)		0..7h & 40..47h
;1x16			0..Fh
;2x16			0..Fh			40..4Fh

;2x16
;ADR_1LCD		EQU    0h		;define adr. of the 1. LCD-bank
;ADR_12LCD		EQU   10h		;define next adr. out of the 1. LCD-bank
;ADR_2LCD		EQU   40h		;define adr. of the 2. LCD-bank
;ADR_22LCD		EQU   50h		;define next adr. out of the 2. LCD-bank

;1x16 (8+8)
ADR_1LCD		EQU    0h		;define adr. of the 1. LCD-bank
ADR_12LCD		EQU   07h		;define next adr. out of the 1. LCD-bank
ADR_2LCD		EQU   40h		;define adr. of the 2. LCD-bank
ADR_22LCD		EQU   48h		;define next adr. out of the 2. LCD-bank

END_CH 			EQU 0ffh		;define for end of a string

;***Pins of HCT138 (IC6)***********************************************************************

LCD_RegWR 	EQU  80h       	;write lcd-register A0=0 ,A1=0
LCD_RegRD 	EQU  81h       	;read  lcd-register A0=1 ,A1=0
LCD_DataWR 	EQU  82h       	;write lcd-data	    A0=0 ,A1=0
LCD_DataRD 	EQU  83h       	;read  lcd-data	    A0=1 ,A1=0

HCT138_Y1	EQU  10010000b
HCT138_Y2	EQU  10100000b
HCT138_Y3	EQU  10110000b
HCT138_Y4	EQU  11000000b
HCT138_Y5	EQU  11010000b
HCT138_Y6	EQU  11100000b
HCT138_Y7	EQU  11110000b

;##################### MAIN Program #######################################

CSEG at 0
	;USING 3                 
	;USING 2                 
	;USING 1                 
	USING 0     	;Bank 0

ORG 0000h
JMP main_start		;jump to program

ORG 0003h        	;extInt0
RETI

ORG 000bh			;timer0
RETI

ORG 0013h			;extInt1
RETI

ORG 001Bh			;timer1
RETI

ORG 0023h			;RS232
RETI
;jmp SIO_INT

;**************************************************************************

main_start:
	MOV SP,  #Stack_First
	MOV DPTR,#0
	MOV ACC, #0
	MOV B,	 #0
	MOV PSW, #0
	MOV PCON,#0
	MOV SCON,#0
	MOV TCON,#0
	MOV TMOD,#0
	MOV IE,  #0
	MOV IP,  #0

	CLR step_walk
	SETB blink_cursor
	CALL lcd_init
	
	call clearLCD				;clear LCD and cursor home	
	
	MOV DPTR,#LCD_TEXT			;alternative code
	SETB step_walk
	CALL lcdstring				;with defined const-string

	MOV A,#44h					;reposition cursor
	CALL setcursor

	MOV lcd_buffer+ 0,#50		;build dynamic LCD-string
	MOV lcd_buffer+ 1,#48
	MOV lcd_buffer+ 2,#48
	MOV lcd_buffer+ 3,#53
	MOV lcd_buffer+ 4,#END_CH	;set end of string
	SETB step_walk
	CALL WriteLCD_Array			;put string to LCD


	MOV P1,#0efh				;Led4 on
	mov a,#0					;clear LCD
	call setcursor

;################## Main - L O O P ########################

MainLoop:
	MOV B,#HCT138_Y2	;read from HCT245
	CALL in_io
	CPL A
	MOV B,#HCT138_Y1	;write to HCT574
	CALL out_io
	CLR LED1			;set LED1,2,3 in case of KEY1,2,3
	JNB KEY1,ReadKey2
	SETB LED1
ReadKey2:
	CLR LED2
	JNB KEY2,ReadKey3	
	SETB LED2
ReadKey3:
	CLR LED3
	JNB KEY3,ReadKey4
	SETB LED3
ReadKey4:			
	JNB KEY4,call_wait	;stop toggle-mode if KEY4 is pressed
	JNB LED4,SetLed4	;toggle LED4		
	CLR LED4
	JMP call_wait
SetLed4:
	SETB LED4	
call_wait:
	CALL wait			;wait a little time
	JMP MainLoop

;----------------------------------------------------------------------
lcdstring:				
	CLR A
	MOVC A,@A+DPTR				;get char
	CJNE A,#END_CH,lcdstring2	;check for end-byte
	JMP lcdstringret
lcdstring2:
	call write_char
	JNB step_walk,lcdstring3
	CALL wait
lcdstring3:
	CALL shortwait
	CALL LCDBusy
	JC lcdstring3				;wait until LCD is ready
	INC DPTR
	JMP lcdstring
lcdstringret:
	CLR step_walk
	RET

;----------------------------------------------------------------------

WriteLCD_Array:
	mov a,tmppointer
	PUSH acc
	MOV tmppointer,#lcd_buffer
WriteLCD1:
	MOV A,@tmppointer
	CJNE A,#END_CH,WriteLCD2		;check for end
	JMP WriteLCDRet
WriteLCD2:
	MOV A,@tmppointer
	call write_char	
	INC tmppointer
	JNB step_walk,WriteLCD3
	CALL wait
WriteLCD3:
	CALL shortwait
	CALL LCDBusy
	JC WriteLCD3					;check if busy
	MOV A,tmppointer
	CLR C
	SUBB A,#lcd_buffer              
	CJNE A,#LCD_BUFMAX,WriteLCD1	;Array_overrun?
WriteLCDRet:
	CLR step_walk
	POP acc
	mov tmppointer,a
	RET

;--------------------------------------------
;put byte to LCD
;acc -> data 
write_char:
	PUSH DPL
	PUSH DPH
	MOV DPL,#LCD_DataWR
	ANL DPL,#0fh
	MOV DPH,#LCD_DataWR
	ANL DPH,#0f0h
	MOVX @DPTR,A
	CALL getCursor
	CJNE A,#ADR_12LCD,wr_ch_end2
	MOV A,#ADR_2LCD					;cursor to 2. line/bank
	CALL setcursor
	JMP wr_char_ret
wr_ch_end2:
	CJNE A,#ADR_22LCD,wr_char_ret
	MOV A,#ADR_1LCD					;cursor to home
	CALL setcursor
	JMP wr_char_ret
wr_char_ret:
	POP DPH
	POP DPL
	RET

;--------------------------------------------
;reads LCD-Contr. Carry=1 if busy
LCDBusy:        
	;anlernative code  -> only short waitcycle
	;call shortwait	;wait a little time
	;clr c		;LCD should be ready
	;ret
	PUSH DPL
	PUSH DPH
	MOV DPL,#LCD_RegRD
	ANL DPL,#0fh
	MOV DPH,#LCD_RegRD
	ANL DPH,#0f0h
	MOVX A,@DPTR
	CLR C
	ANL A,#80h
	JZ LCDBusyRet
	SETB C
LCDBusyRet:
	POP DPH
	POP DPL
	RET
	
;--------------------------------------------
; set cursor in LCD-module
; in acc
setcursor:
	PUSH DPL
	PUSH DPH
	MOV DPL,#LCD_RegWR
	ANL DPL,#0fh
	MOV DPH,#LCD_RegWR
	ANL DPH,#0f0h
	ORL A,#80h				;set command-bit
	MOVX @DPTR,A			;set cursor
	POP DPH
	POP DPL
	RET

;--------------------------------------------
;read RAM-addr of LCD and return it in acc (0-127)
;out acc
getCursor:
	PUSH DPL
	PUSH DPH
	MOV DPL,#LCD_RegRD
	ANL DPL,#0fh
	MOV DPH,#LCD_RegRD
	ANL DPH,#0f0h
	MOVX A,@DPTR
	ANL A,#7Fh				;clear busy-flag
	POP DPH
	POP DPL
	RET
;--------------------------------------------
; set cursor home and clear LCD-module
clearLCD:
	PUSH DPL
	PUSH DPH
	MOV A,#1h               ;homecursor and clear

	MOV DPL,#LCD_RegWR
	ANL DPL,#0fh
	MOV DPH,#LCD_RegWR
	ANL DPH,#0f0h
	MOVX @DPTR,A
clear_busy:
	CALL shortwait
	CALL LCDBusy
	JC clear_busy
	POP DPH
	POP DPL
	RET

;--------------------------------------------
; wait a little bit
shortwait:
	PUSH ACC
	MOV A,#0
shortwait0:
	NOP
	NOP
	NOP
	DJNZ ACC,shortwait0
	POP ACC
	RET

;--------------------------------------------
; wait a little bit longer 
wait:
	MOV A,#waitconst
wait0:
	CALL shortwait
	DJNZ ACC,wait0
	RET

;--------------------------------------------
;acc -> data 
;b   -> IO-high-adresse
out_io:
	PUSH DPL
	PUSH DPH
	MOV DPL,B
	ANL DPL,#0fh
	MOV DPH,B
	ANL DPH,#0f0h
	MOVX @DPTR,A
	POP DPH
	POP DPL
	RET

;--------------------------------------------
;return data in acc 
;b   -> IO-high-adresse
in_io:
	PUSH DPL
	PUSH DPH
	MOV DPL,B
	ANL DPL,#0fh
	MOV DPH,B
	ANL DPH,#0f0h
	MOVX A,@DPTR
	POP DPH
	POP DPL
	RET

;-----------------------------------------------------------------
;init LCD-modul
lcd_init:
	CALL shortwait
	MOV A,#00111000b			;8bit-mode one line 5x7dots
	MOV B,#LCD_RegWR
	LCALL out_io
	LCALL out_io
	CALL shortwait
	JB blink_cursor,b_cursor	
	MOV A,#00001100b      		;LCD on no Cursor
	JMP after_cur
b_cursor:
	MOV A,#00001101b      		;LCD on no Cursor
after_cur:
	MOV B,#LCD_RegWR
	LCALL out_io
	LCALL shortwait
	MOV A,#00000001b			;clear lcd
	MOV B,#LCD_RegWR
	LCALL out_io
	LCALL shortwait
	MOV A,#00000110b			;set corsor-direction auto-dec
	MOV B,#LCD_RegWR
	LCALL out_io
	LCALL shortwait
	CLR blink_cursor
	RET

;****************************************************************

LCD_EMPTY:     DB '                ',END_CH
LCD_TEXT:      DB 'FrydaySoft c????',END_CH

END