// SDCC debug example with NOICE for controller-board (www.frydaysoft.de)
// a) upload the NOICE-monitor (Mon8051.rom)
//    - use asm_dbg project or uplaod Mon8051.rom with external tool "nt8051.exe"
//
// b) compile this debug example
//    some important notes:
//    - linked to 0x100, because NOICE start at 0x100 (default) and can not write to adresses below 0x100
//    - Makefile copy jump pattern (0-0xFF) before compiled code --> dbg_test.bin
//    - jump pattern jumps to 0x7000 (NOICE-monitor) after upload
//    - deligate interrupts in jump pattern if you use interrupts (--> not tested with NOICE !!!)
// 
// c) open NoIce-Debugger with serial protocol 19200baud
//
// d) start debug session with NOICE
//	  - run <File-->load> dbg_test.ihx (this 8051 code) with NOICE (over RS232) to the NOICE-monitor
//    - run <File-->play> dbg_test.noi with NOICE
//    - if sourcefile not loaded into NOICE --> use "Q"-Button (Select source file) if enabled after 'play'-command
//    - set breakpoint in main() and run code
// e) do some tests / changes in debug-application go on with b)

#include "delay_ms.h"


//memory port of HC574
xdata at 0x9000 unsigned char hc574_chip;

xdata at 0x8000 unsigned char p82c55_port_a;
xdata at 0x8001 unsigned char p82c55_port_b;
xdata at 0x8002 unsigned char p82c55_port_c;
xdata at 0x8003 unsigned char p82c55_config;

struct led_pattern_struct {
	unsigned char pattern;
	unsigned char delay;
};

code struct led_pattern_struct led_table[] = {
{0x7F, /* 01111111 */ 90},
{0xBF, /* 10111111 */ 80},
{0xDF, /* 11011111 */ 60},
{0xEF, /* 11101111 */ 50},
{0xF7, /* 11110111 */ 40},
{0xFB, /* 11111011 */ 30},
{0xFD, /* 11111101 */ 20},
{0xFE, /* 11111110 */ 30},
{0xFD, /* 11111101 */ 40},
{0xFB, /* 11111011 */ 50},
{0xF7, /* 11110111 */ 60},
{0xEF, /* 11101111 */ 70},
{0xDF, /* 11011111 */ 80},
{0xBF, /* 10111111 */ 90},
{255,0}};


/*
// example to set ISR-function to defined code address
void reset_isr(void) _naked
{ 
	_asm
    .area   CODE0   (ABS)   ;program area CODE1 is ABSOLUTE
    .org    0H000           ;set location to 0x0000 (reset vector)
	ljmp 0x7000
    .org    0H100           ;set location to 0x0100 (default start for NOICE)
	.area HOME    (CODE)
	_endasm;
} 
*/


unsigned char _sdcc_external_startup ()
{
    return 0;
}


void main()
{
	code struct led_pattern_struct *p=led_table;
	unsigned char x=0;

	p82c55_config = 128;	/* all ports outputs */

	while (1) 
	{
		if (p->delay > 0) 
		{

			p82c55_port_a = p->pattern;
			hc574_chip = p->pattern;

			for (x=0; x<10;x++)
 			 delay_ms(p->delay);

			p++;
		}
		 else 
		{
			p = led_table;
		}
	}
}


