// flasher software for Flash-Module running on the controller-board (www.frydaysoft.de)
// extext flash data at RAM address CODE_INFO (0x1000)
// build uploadfile external
//   - this flasher software
//   - on CODE_INFO some information for programming (length, source address, destination address)
//   - data for flash
// You can use Windows-Application from "flasher2.zip" and rebuild "flasher.exe" with C-Array of this 8081 code


#include "delay_ms.h"
#include "sio.h"

sfr at 0x90 ledP1;

#define divTimer 250		// timer1 -> divTimer=256-fosz/(Baud*12*16)

// #define TEST 0x8000
#define CODE_INFO 0x1000		

unsigned char ManCode=0;
unsigned char DevCode=0;	

#define READ_FLASH 0
#define READ_RAM 1
#define READ_HIRAM 2

// *******************************************************************************************
// init SIO for 9600 at 11.0592 MHz
unsigned char _sdcc_external_startup ()
{
	_asm
	mov P1,#0x00
	init_com:
		orl PCON,#0x80		; normal or faster processing
		orl SCON,#0x50		; SCON-Mode 1 (8Bit-UART) and REN 
		orl TCON,#0x40		; set TR1 ->use TIMER1 (xxx1xxxx)
		orl TMOD,#0x20		; set 8Bit-auto-reload-TIMER0  (00100000)
		mov TH1,#divTimer	; init Timer1 for Baud 9600 at 11.0592 MHz
		mov TL1,#divTimer
	
		setb SM2
		clr TF1
		setb TR1
		clr ET1
		clr RI        		
		clr PS		   		;lower priority
		setb ES
	_endasm;
    return 0;
}

// *******************************************************************************************
void ShowResult(unsigned char errCode)
{
	unsigned char x=errCode;
	unsigned char dir=0x1;
	
	if (errCode==0)
	{
		x=0x10;
		ledP1=~x;
		while(1)
		{	// run forever in LED-loop
			_asm
			clr c
			_endasm;
			if (dir) x=x<<1;
			 else x=x>>1;
			if (x==0x80) dir=0;
			if (x==0x10) dir=1;		 
			delay_ms(500);
			ledP1=~x;	
		}			
	}
		
	while(1)
	{	// run forever in LED-loop
		ledP1=~x;
		delay_ms(500);	
		ledP1=x;
		delay_ms(500);		
	}	
}

// *******************************************************************************************
unsigned char readRAM(int ptr)
{
	ptr;
	_asm
	push acc
	mov a,dph
	clr acc.7				; read only in RAM/ROM-window !!!
	mov dph,a
	clr a
	movc a,@a+dptr			; /PSEN=0
	mov dpl,a				;return the byte
	pop acc
	ret
	_endasm;	
}

// *******************************************************************************************
unsigned char readHI_RAM(int ptr)
{
	ptr;
	_asm
	push acc
	mov a,dph
	setb acc.7				; read only in (upper) ROM
	mov dph,a
	clr a
	movc a,@a+dptr			; A15=1  &  /PSEN=0
	mov dpl,a				;return the byte
	pop acc
	ret
	_endasm;	
}

// *******************************************************************************************
void writeByte(int ptr, unsigned char c)
{
	ptr;					// dptr
	c;						// _writeByte_PARM_2
	_asm
	push acc
	push dph
	clr P1.0		
	mov a,dph
	jnb acc.7,writeByteSet
	setb P1.0				;set A15 for Flash
writeByteSet:
	setb acc.7				;set A15 for IC-frame
	mov dph,A
	mov A,_writeByte_PARM_2	
	movx @dptr,A			; /wr=0
	clr P1.0
	pop dph
	pop acc
	ret	
	_endasm;	
}

// *******************************************************************************************
unsigned char readByte(int ptr)
{
	ptr;
	_asm
	push acc
	push dph	
	clr P1.0
	mov a,dph
	jnb acc.7,readByteSet
	setb P1.0				;set A15 for Flash
readByteSet:
	setb acc.7				;set A15 for IC-frame
	mov dph,A
	clr a					
	movc a,@a+dptr			; /PSEN=0
	mov dpl,a				;return the byte
	clr P1.0		
	pop dph
	pop acc
	ret
	_endasm;	
}

// *******************************************************************************************
unsigned char waitBusy(int adr) 
{
	unsigned char prev_state=0;
    unsigned char state=0;
    int timeout=50;

    prev_state = readByte(adr) & 0x40;
    while (1) 
    {
        state = readByte(adr) & 0x40;
        if (prev_state == state) {
            break;
        }
        if (--timeout == 0) {
            puts("!!! Timeout !!!\r\n\0");
            return 1;
        }
        prev_state = state;
        delay_ms(2);
    }
    return 0;
}

// *******************************************************************************************
// *******************************************************************************************
// *******************************************************************************************
#define ATMEL_FLASH_29C010 0xD5
#define ATMEL_FLASH_29C512 0x5D
#define MANUFACTURER_CODE 0x1F


// *******************************************************************************************
unsigned char checkFlashID()
{
	unsigned char ret=1;
	writeByte(0x5555, 0xaa);	writeByte(0x2aaa, 0x55);
	writeByte(0x5555, 0x90);	// get ID	

	delay_ms(10);
	
	ManCode=readByte(0x0000);	DevCode=readByte(0x0001);
	
	writeByte(0x5555, 0xaa);	writeByte(0x2aaa, 0x55);
	writeByte(0x5555, 0xF0);	// exit "get-ID"-state
	
	delay_ms(10);
	puts("Manufacturer: 0x\0");
	putHex(ManCode);			newLine();	
	puts("Flash-ID: 0x\0");			putHex(DevCode);			newLine();	
		
	if (ManCode==MANUFACTURER_CODE)
	{
		puts("--> found an ATMEL \0");
		switch(DevCode)
		{	
			case ATMEL_FLASH_29C512:	puts(" 29C512\r\n\0");	ret=0;	break;
			case ATMEL_FLASH_29C010:	puts(" 29C010\r\n\0");	ret=0;	break;			
		};
	}
	if (ret) puts("!!! unknown Flash-ID !!!\r\n\0");
	return ret;
}

// *******************************************************************************************
void verifyData(int sourcePtr, int destPtr, int len)
{
	unsigned char valSource=0;
	unsigned char valDest=0;
	
	puts("\r\nverify 0x\0");
	valSource=len>>8;		putHex(valSource);		valSource=len;		putHex(valSource);
	puts(" bytes at 0x\0");
	valDest=destPtr>>8;		putHex(valDest);	valDest=destPtr;	putHex(valDest);
	
	while(len)
	{
		valSource=readRAM(sourcePtr);
		valDest=readByte(destPtr);
		if (valSource!=valDest )
		{
			puts("\r\n Verify-Error: read 0x\0");
			putHex(valDest);
			puts(" from 0x\0");		
			valDest=destPtr>>8;
			putHex(valDest);
			valDest=destPtr;
			putHex(valDest);	
			puts(" != 0x\0");
			putHex(valSource);		
			ShowResult(0x0F);
		}		
		sourcePtr++;
		destPtr++;
		len--;
	}
	puts(" ...ok\0");	
}

// *******************************************************************************************
// write 128 Bytes from ptr
void writeBlock(int sourcePtr, int destPtr, unsigned char len, unsigned char set)
{
	unsigned char cnt=0;
	unsigned char val=0;
	int adr=sourcePtr;
	int adr2=destPtr&0xFF80;
	
	puts("\r\nwrite \0");
	putDez(len);
	puts(" bytes to 0x\0");
	val=destPtr>>8;		putHex(val);		val=destPtr;		putHex(val);
	
	val=0;
	if(set==1)
	{
		writeByte(0x5555,0xAA);		writeByte(0x2aaa,0x55);
		writeByte(0x5555,0xA0);		// enable protection and write 128 bytes
	}
	if(set==0)
	{
		writeByte(0x5555,0xAA);		writeByte(0x2aaa,0x55);
		writeByte(0x5555,0x80);		writeByte(0x5555,0xAA);
		writeByte(0x2aaa,0x55);
		writeByte(0x5555,0x20);		// disable protection and write 128 bytes	
	}
	
	if(set<2)
	{	// write 128 dummies
		cnt=0;
		while(cnt<128)
		{
			writeByte(adr2, 0xFF);
			adr2++;			cnt++;
		}
		adr2--;		waitBusy(adr2);			// let the uc write all data
	}
	
	adr2=destPtr&0xFF80;
	
	while(cnt<128)
	{
		val=readRAM(adr);
		if (cnt>=len)			// DATA-buffer smaller than 128 bytes ?
		{
			val=0xFF;
		}
		writeByte(adr2, val);
		adr++;		adr2++;		cnt++;
	}
	delay_ms(10);				// let the uc write all data	
	
	adr=sourcePtr;
	adr2=destPtr&0xFF80;
	while(cnt<128)				// verify
	{
		val=readRAM(adr);
		if (cnt>=len)			// DATA-buffer smaller than 128 bytes ?
		{
			val=0xFF;
		}
		if (val!=readByte(adr2) )
		{
			puts("  Error at: 0x\0");		
			val=adr2>>8;			putHex(val);
			val=adr2;			putHex(val);
			puts(": \0");
			ShowResult(0x0F);
		}		
		adr++;
		adr2++;
		cnt++;
	
	}
	puts(" ...ok\0");			
}

// *******************************************************************************************
// read 256 Bytes from ptr		memType==0 --> from Flash  else from RAM
void readBlock(int ptr, unsigned char end, unsigned char memType)
{
	unsigned char x=0;
	unsigned char cnt=0;
	int adr=ptr;
	do
	{
		if (cnt%16==0) 
		{
			newLine();
			x=adr>>8;		putHex(x);		x=adr;		putHex(x);
			puts(": \0");
		}
		
		switch (memType)
		{
			case READ_FLASH:	x=readByte(adr); break;
			case READ_RAM:	x=readRAM(adr); break;
			default: x=readHI_RAM(adr);break;
		};		
		putHex(x);
		cout(' ');

		adr++;
		cnt++;
	}while(cnt!=end);
	newLine();			
}

// *******************************************************************************************
void main()
{
	unsigned char x=0x10;
	int source=CODE_INFO+5;
	int dest=0x0;
	int len=0;
	int partLen=0;
	
	newLine();
	puts("\r\ncheck FlashModul...\0");
	newLine();	

	x=checkFlashID();
	x=1;	// ignore ID-check	
	if (x==1)	
	{
		len=readRAM(CODE_INFO)*256+readRAM(CODE_INFO+1);
		x=readRAM(CODE_INFO+2);	// for future use
		dest=readRAM(CODE_INFO+3)*256+readRAM(CODE_INFO+4);
			
		puts("write DATA to: 0x\0");
		x=dest>>8;		putHex(x);		x=dest;		putHex(x);	
		puts("     Length: 0x\0");
		x=len>>8;		putHex(x);		x=len;		putHex(x);		newLine();	
				
		while(len)
		{
			if (len>128)
			{	partLen=128; }
			else { partLen=len; }
			writeBlock(source, dest, partLen, 2);	// 0: disable protection	1: enable protection	2: write 128 bytes
			source+=partLen;
			dest+=partLen;
			len-=partLen;
		}
		newLine();			
		
		len=readRAM(CODE_INFO)*256+readRAM(CODE_INFO+1);
		x=readRAM(CODE_INFO+2);	// for future use
		dest=readRAM(CODE_INFO+3)*256+readRAM(CODE_INFO+4);
		source=CODE_INFO+5;
		verifyData(source,dest,len);
		ShowResult(0);
	}
}
