#include "c:\ide51\c_bin\include\mcs51\8051.h"
#include "lcd_sio.h"


#define LCD_RegWR 	0x8000       	//write lcd-register A0=0 ,A1=0
#define LCD_RegRD 	0x8001       	//read  lcd-register A0=1 ,A1=0
#define LCD_DataWR 	0x8002       	//write lcd-data	    A0=0 ,A1=0
#define LCD_DataRD 	0x8003       	//read  lcd-data	    A0=1 ,A1=0


void shortWait(void)_naked
{
	_asm
	shortwait:
		push acc
		mov a,#0
	shortwait0:
		nop
		nop
		djnz acc,shortwait0
		pop acc
		ret
	_endasm;
}

int isLcdBusy(void)_naked
{
	_asm
LCDBusy:	
		push dph
		push acc
		mov dptr,#LCD_RegRD
		movx a,@dptr
		anl a,#0x80
		jz isLcdBusy_0
		orl a,#0x01
isLcdBusy_0:		
		anl a,#0x01
		pop dph
		mov dpl,a
		pop acc
		ret
	_endasm;
}

void setLCD_Cursor(unsigned char pos)_naked
{
	pos;
	_asm
		push dph
		push acc
		mov a,dpl
		mov dptr,#LCD_RegWR
		orl a,#0x80				;set command-bit
		movx @dptr,a			;set cursor
		pop acc
		pop dph
		ret
	_endasm;
}

unsigned char getLCD_Cursor(void)_naked
{
	_asm
		push dph
		push acc
		mov a,dpl
		mov dptr,#LCD_RegRD
		movx a,@dptr
		anl a,#0x7F				;clear busy-flag
		mov dpl,a
		pop acc
		pop dph
		ret
	_endasm;
}

void clearLCD(void)_naked
{
	_asm
		push dph
		push dpl
		push acc

		mov a,#1	            ;homecursor and clear

		mov dptr,#LCD_RegWR
		movx @dptr,a
	clear_busy:
		acall shortwait
		acall LCDBusy
		mov a,dpl
		jnz clear_busy
		pop acc
		pop dpl
		pop dph
		ret
	_endasm;
}


void initLCD(unsigned char lcdType)_naked
{
	lcdType;
	_asm
		push b
		mov b,dpl
		mov dptr,#LCD_RegWR
		mov A,b						; init with lcdType
		movx @dptr,A
		acall shortwait
		acall shortwait	
		movx @dptr,A				; init with lcdType again
		acall shortwait
		acall shortwait	
	
		mov A,#0xD					;  LCD on and show Cursor
		;mov A,#0xC					;  LCD on and hide Cursor
	
		mov dptr,#LCD_RegWR
		movx @dptr,A
		acall shortwait
		mov A,#0x1					; clear lcd
		movx @dptr,A
		acall shortwait
		mov A,#0x5					; set corsor-direction auto-dec
		movx @dptr,A
		acall shortwait
		pop b
		ret
	_endasm;
}

void charLCD(unsigned char ch)_naked
{
	ch;
	_asm
	charLCD:
		push dph
		mov a,dpl
		mov dptr,#LCD_DataWR
		movx @dptr,A
		acall shortwait	
		pop dph
		ret
	_endasm;
}


// --- SIO -------------------------------------------------------------
void initCom(unsigned char baud)_naked
{
	baud;
	_asm
		orl PCON,#0x80		; normal or faster processing
		orl SCON,#0x50		; SCON-Mode 1 (8Bit-UART) and REN 
		orl TCON,#0x40		; set TR1 ->use TIMER1 (xxx1xxxx)
		orl TMOD,#0x20		; set 8Bit-auto-reload-TIMER0  (00100000)
		mov TH1,dpl			; init Timer1 for Baud 9600 at 11.0592 MHz
		mov TL1,dpl
	
		setb SM2
		clr TF1
		setb TR1
		clr ET1
		clr RI        		
		clr PS		   		;lower priority
		setb ES
		ret
	_endasm;
}

char cin(void) _naked
{
	_asm
CIN:
	 jnb RI,CIN	
 	 clr RI
     mov dpl,SBUF
     ret
	_endasm;
}

void cout(unsigned char c) _naked
{
	c;
	_asm  
	 mov SBUF,dpl
	 clr TI
COUT:		
	 jnb TI,COUT
	 ret
	_endasm;
}

void newLine(void)_naked
{
	_asm
		push dpl
	_endasm;

	cout(10);
	cout(13);
	_asm
		pop dpl
		ret
	_endasm;
}

void puts(unsigned char *ptr)
{
	ptr;
	if(!ptr) return;
	while(*ptr)			// print until null-terminated
	{
		cout(*ptr);
		ptr++;
	}
}

void putBin(unsigned char c)
{
	char mask=0x80;
	while(mask)
	{
		if (c&mask) cout('1'); 
		 else { cout('0'); }
		mask=mask>>1;			// shift mask
		mask=mask&0x7F;			// clear shifted carry
	};
}

void putHex(unsigned char c)
{
	char part=c;
	part=part>>4;
	part=part&0x0F;
	//HI
	if (part>9) part+=7;	// a..f
	cout(part+48);			// ASCII: 0..9 a..f
	//LO
	part=c&0x0F;
	if (part>9) part+=7;	// a..f
	cout(part+48);			// ASCII: 0..9 a..f
}
