// timer-interrupt-example  			www.frydaysoft.de
// count value in timer interrupt and 
// toggle LED in main depending on counter value

//******************************************************************************
#include "io.h"
#include "timer.h"


//onboard LEDs on Port0
sfr at 0x90 ledP1;

#define divTimer 250			// timer1 for SIO -> divTimer=256-fosz/(Baud*12*16)

//counter for Timer0
data unsigned char timerCalls;	// --> accass per ANSI-C and asm


// FFFFh - (XTAL/12) / 10ms   	;at 11.0592 MHz  --> 56320 = DC00h
#define TH0Value 0xDC           //16BIT-value for 10ms-timer
#define TL0Value 0x00 
#define Cnt_T0 50	

#define HCT138_Y1	0x9000		//HC574
#define HCT138_Y2	0xA000
#define HCT138_Y3	0xB000
#define HCT138_Y4	0xC000
#define HCT138_Y5	0xD000
#define HCT138_Y6	0xE000
#define HCT138_Y7	0xF000			

//******************************************************************************
//vector 0x0B
void timer_isr (void) interrupt 1
{

	_asm
	clr TR0				//stop T0
	_endasm;
	
	timerCalls++;		//this timervariable is used in the application

	_asm
	mov TL0,#TL0Value	//reinit T0
	mov TH0,#TH0Value 
	setb TR0			//run T0
	_endasm;
}

// *******************************************************************************************
// init SIO for 9600 at 11.0592 MHz
unsigned char _sdcc_external_startup ()
{
	unsigned char x=0;
	_asm
	mov P1,#0x00
		mov TCON,#0			//no Intr.
	init_com:
		orl PCON,#0x80		// normal or faster processing
		orl SCON,#0x50		// SCON-Mode 1 (8Bit-UART) and REN 
		orl TCON,#0x40		// set TR1 ->use TIMER1 (xxx1xxxx)
		orl TMOD,#0x20		// set 8Bit-auto-reload-TIMER0  (00100000)
		mov TH1,#divTimer	// init Timer1 for Baud 9600 at 11.0592 MHz
		mov TL1,#divTimer
	
		setb SM2
		clr TF1
		setb TR1
		clr ET1
		clr RI        		
		clr PS		   		// lower priority
//		setb ES
	init_T0:
		clr TR0				// stop T0
		anl TMOD,#0xF0		// 8bits: XXXX . gate C/T MODE.1 MODE.0
		orl TMOD,#0x01		// Gate0=enabled and as Timer (0x01->16-bit 0x02->8bit reload)
		mov TL0,#TL0Value
		mov TH0,#TH0Value 
		clr TF0
		setb PT0			//higher priority
		setb ET0			//enable T0-intr.
		setb EA				//enable all Intr.
		mov _timerCalls,#0
	_endasm;
	

	_asm;	
		setb TR0			//run T0	
	_endasm;
	return 0;
}


//******************************************************************************
//main function
void main()
{
	unsigned char by_dummy=0;

	outp(HCT138_Y1, 0xFF);					// set all LEDs on IC8 off

	while(1)								// MAIN-LOOP
	{
		if ( timerCalls > 100 )
		{
			_asm
			clr TR0							//stop T0
			mov _timerCalls,#0				//reset counter value
			setb TR0						//run T0
			_endasm;

			_asm;	
			cpl P1.4						// toggle led with asm
			_endasm;
			ledP1 ^= 0x80;					// toggle led with ANSI-C
			outp(HCT138_Y1, ~inp(HCT138_Y2));
		}
	}
}

//******************************************************************************
